pub struct Consensus<N: Network> { /* private fields */ }Expand description
Wrapper around BFT that adds additional functionality, such as a mempool.
Consensus acts as a rate limiter to prevents workers in BFT from being overloaded. Each worker maintains a ready queue (which is essentially also a mempool), but verifies transactions/solutions before enqueuing them. Consensus only passes more transactions/solutions to the BFT layer if its ready queues are not already full.
Implementations
Sourceimpl<N: Network> Consensus<N>
impl<N: Network> Consensus<N>
Sourcepub async fn new(
account: Account<N>,
ledger: Arc<dyn LedgerService<N>>,
block_sync: Arc<BlockSync<N>>,
ip: Option<SocketAddr>,
trusted_validators: &[SocketAddr],
trusted_peers_only: bool,
storage_mode: StorageMode,
node_data_dir: NodeDataDir,
ping: Arc<Ping<N>>,
dev: Option<u16>,
) -> Result<Self>
pub async fn new( account: Account<N>, ledger: Arc<dyn LedgerService<N>>, block_sync: Arc<BlockSync<N>>, ip: Option<SocketAddr>, trusted_validators: &[SocketAddr], trusted_peers_only: bool, storage_mode: StorageMode, node_data_dir: NodeDataDir, ping: Arc<Ping<N>>, dev: Option<u16>, ) -> Result<Self>
Initializes a new instance of consensus.
pub fn contains_transaction(&self, transaction_id: &N::TransactionID) -> bool
Sourceimpl<N: Network> Consensus<N>
impl<N: Network> Consensus<N>
Sourcepub fn num_unconfirmed_transmissions(&self) -> usize
pub fn num_unconfirmed_transmissions(&self) -> usize
Returns the number of unconfirmed transmissions in the BFT’s workers (not in the mempool).
Sourcepub fn num_unconfirmed_ratifications(&self) -> usize
pub fn num_unconfirmed_ratifications(&self) -> usize
Returns the number of unconfirmed ratifications in the BFT’s workers (not in the mempool).
Sourcepub fn num_unconfirmed_solutions(&self) -> usize
pub fn num_unconfirmed_solutions(&self) -> usize
Returns the number unconfirmed solutions in the BFT’s workers (not in the mempool).
Sourcepub fn num_unconfirmed_transactions(&self) -> usize
pub fn num_unconfirmed_transactions(&self) -> usize
Returns the number of unconfirmed transactions.
Sourceimpl<N: Network> Consensus<N>
impl<N: Network> Consensus<N>
Sourcepub fn unconfirmed_transmission_ids(
&self,
) -> impl '_ + Iterator<Item = TransmissionID<N>>
pub fn unconfirmed_transmission_ids( &self, ) -> impl '_ + Iterator<Item = TransmissionID<N>>
Returns the unconfirmed transmission IDs.
Sourcepub fn unconfirmed_transmissions(
&self,
) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>
pub fn unconfirmed_transmissions( &self, ) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>
Returns the unconfirmed transmissions.
Sourcepub fn unconfirmed_solutions(
&self,
) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>
pub fn unconfirmed_solutions( &self, ) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>
Returns the unconfirmed solutions.
Sourcepub fn unconfirmed_transactions(
&self,
) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>
pub fn unconfirmed_transactions( &self, ) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>
Returns the unconfirmed transactions.
Sourceimpl<N: Network> Consensus<N>
impl<N: Network> Consensus<N>
Sourcepub fn worker_transmission_ids(
&self,
) -> impl '_ + Iterator<Item = TransmissionID<N>>
pub fn worker_transmission_ids( &self, ) -> impl '_ + Iterator<Item = TransmissionID<N>>
Returns the worker transmission IDs.
Sourcepub fn worker_transmissions(
&self,
) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>
pub fn worker_transmissions( &self, ) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>
Returns the worker transmissions.
Sourcepub fn worker_solutions(
&self,
) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>
pub fn worker_solutions( &self, ) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>
Returns the worker solutions.
Sourcepub fn worker_transactions(
&self,
) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>
pub fn worker_transactions( &self, ) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>
Returns the worker transactions.
Sourceimpl<N: Network> Consensus<N>
impl<N: Network> Consensus<N>
Sourcepub fn inbound_transmission_ids(
&self,
) -> impl '_ + Iterator<Item = TransmissionID<N>>
pub fn inbound_transmission_ids( &self, ) -> impl '_ + Iterator<Item = TransmissionID<N>>
Returns the transmission IDs in the inbound queue.
Sourcepub fn inbound_transmissions(
&self,
) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>
pub fn inbound_transmissions( &self, ) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>
Returns the transmissions in the inbound queue.
Sourcepub fn inbound_solutions(
&self,
) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>
pub fn inbound_solutions( &self, ) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>
Returns the solutions in the inbound queue.
Sourcepub fn inbound_transactions(
&self,
) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>
pub fn inbound_transactions( &self, ) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>
Returns the transactions in the inbound queue.
Sourceimpl<N: Network> Consensus<N>
impl<N: Network> Consensus<N>
Sourcepub async fn add_unconfirmed_solution(
&self,
solution: Solution<N>,
) -> Result<()>
pub async fn add_unconfirmed_solution( &self, solution: Solution<N>, ) -> Result<()>
Adds the given unconfirmed solution to the memory pool, which will then eventually be passed to the BFT layer for inclusion in a batch.
Sourcepub async fn add_unconfirmed_transaction(
&self,
transaction: Transaction<N>,
) -> Result<()>
pub async fn add_unconfirmed_transaction( &self, transaction: Transaction<N>, ) -> Result<()>
Adds the given unconfirmed transaction to the memory pool, which will then eventually be passed to the BFT layer for inclusion in a batch.
Sourceimpl<N: Network> Consensus<N>
impl<N: Network> Consensus<N>
Sourcepub async fn start_consensus_handlers(&self) -> Result<()>
pub async fn start_consensus_handlers(&self) -> Result<()>
Starts the BFT and consensus handlers.
This consumes the deferred state stored during construction and must only be invoked once, after the node has finished syncing from the CDN, so that no blocks are committed during the initial sync.