Consensus

Struct Consensus 

pub struct Consensus<N: Network> { /* private fields */ }
Wrapper around BFT that adds additional functionality, such as a mempool.

Consensus acts as a rate limiter to prevents workers in BFT from being overloaded. Each worker maintains a ready queue (which is essentially also a mempool), but verifies transactions/solutions before enqueuing them. Consensus only passes more transactions/solutions to the BFT layer if its ready queues are not already full.

Implementations

Source

impl<N: Network> Consensus<N>

pub async fn new( account: Account<N>, ledger: Arc<dyn LedgerService<N>>, block_sync: Arc<BlockSync<N>>, ip: Option<SocketAddr>, trusted_validators: &[SocketAddr], trusted_peers_only: bool, storage_mode: StorageMode, node_data_dir: NodeDataDir, ping: Arc<Ping<N>>, dev: Option<u16>, ) -> Result<Self>

Initializes a new instance of consensus.

pub const fn bft(&self) -> &BFT<N>

Returns the underlying BFT struct.

pub fn contains_transaction(&self, transaction_id: &N::TransactionID) -> bool

Source

impl<N: Network> Consensus<N>

pub fn num_unconfirmed_transmissions(&self) -> usize

Returns the number of unconfirmed transmissions in the BFT’s workers (not in the mempool).

pub fn num_unconfirmed_ratifications(&self) -> usize

Returns the number of unconfirmed ratifications in the BFT’s workers (not in the mempool).

pub fn num_unconfirmed_solutions(&self) -> usize

Returns the number unconfirmed solutions in the BFT’s workers (not in the mempool).

pub fn num_unconfirmed_transactions(&self) -> usize

Returns the number of unconfirmed transactions.

Source

impl<N: Network> Consensus<N>

pub fn unconfirmed_transmission_ids( &self, ) -> impl '_ + Iterator<Item = TransmissionID<N>>

Returns the unconfirmed transmission IDs.

pub fn unconfirmed_transmissions( &self, ) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>

Returns the unconfirmed transmissions.

pub fn unconfirmed_solutions( &self, ) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>

Returns the unconfirmed solutions.

pub fn unconfirmed_transactions( &self, ) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>

Returns the unconfirmed transactions.

Source

impl<N: Network> Consensus<N>

pub fn worker_transmission_ids( &self, ) -> impl '_ + Iterator<Item = TransmissionID<N>>

Returns the worker transmission IDs.

pub fn worker_transmissions( &self, ) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>

Returns the worker transmissions.

pub fn worker_solutions( &self, ) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>

Returns the worker solutions.

pub fn worker_transactions( &self, ) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>

Returns the worker transactions.

Source

impl<N: Network> Consensus<N>

pub fn inbound_transmission_ids( &self, ) -> impl '_ + Iterator<Item = TransmissionID<N>>

Returns the transmission IDs in the inbound queue.

pub fn inbound_transmissions( &self, ) -> impl '_ + Iterator<Item = (TransmissionID<N>, Transmission<N>)>

Returns the transmissions in the inbound queue.

pub fn inbound_solutions( &self, ) -> impl '_ + Iterator<Item = (SolutionID<N>, Data<Solution<N>>)>

Returns the solutions in the inbound queue.

pub fn inbound_transactions( &self, ) -> impl '_ + Iterator<Item = (N::TransactionID, Data<Transaction<N>>)>

Returns the transactions in the inbound queue.

Source

impl<N: Network> Consensus<N>

pub async fn add_unconfirmed_solution( &self, solution: Solution<N>, ) -> Result<()>

Adds the given unconfirmed solution to the memory pool, which will then eventually be passed to the BFT layer for inclusion in a batch.

pub async fn add_unconfirmed_transaction( &self, transaction: Transaction<N>, ) -> Result<()>

Adds the given unconfirmed transaction to the memory pool, which will then eventually be passed to the BFT layer for inclusion in a batch.

Source

impl<N: Network> Consensus<N>

pub async fn start_consensus_handlers(&self) -> Result<()>

Starts the BFT and consensus handlers.

This consumes the deferred state stored during construction and must only be invoked once, after the node has finished syncing from the CDN, so that no blocks are committed during the initial sync.

pub async fn shut_down(&self)

Shuts down the consensus and BFT layers.